@@ -20,6 +20,8 @@ from . import conversion
 
 S = TypeVar('S')
 T = TypeVar('T')
+T_co = TypeVar('T_co')
+T_con = TypeVar('T_con')
 LogicHandler = Callable[[T, List[Any]], Awaitable[None]]
 
 logger = logging.getLogger(__name__)
@@ -84,8 +86,8 @@ class ConnectableWrapper(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
         pass
 
 
-class Writable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
-    async def write(self, value: T, origin: Optional[List[Any]] = None) -> None:
+class Writable(Connectable[T_con], Generic[T_con], metaclass=abc.ABCMeta):
+    async def write(self, value: T_con, origin: Optional[List[Any]] = None) -> None:
         """
         Asynchronous coroutine to update the object with a new value
 
@@ -115,7 +117,7 @@ class Writable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
         await self._write(value, origin)
 
     @abc.abstractmethod
-    async def _write(self, value: T, origin: List[Any]) -> None:
+    async def _write(self, value: T_con, origin: List[Any]) -> None:
         """
         Abstract internal method containing the actual implementation-specific write-logic.
 
@@ -124,6 +126,7 @@ class Writable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
 
         Please make sure that your `_write` implementation awaits the processing of the value update by the next
         stateful object/system before returning:
+
         - On a Variable and similar stateful re-publishing objects, which use `_stateful_publishing = True`, await the
           storing and `_publish()`-ing of the new value
         - On an Expression and similar stateless re-publishing objects, use `_publish_and_wait()` instead of
@@ -140,9 +143,9 @@ class Writable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
         pass
 
 
-class Readable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
+class Readable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
     @abc.abstractmethod
-    async def read(self) -> T:
+    async def read(self) -> T_co:
         pass
 
 
@@ -150,16 +153,16 @@ class UninitializedError(RuntimeError):
     pass
 
 
-class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
+class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
     _stateful_publishing: bool = False
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._subscribers: List[Tuple[Writable[S], Optional[Callable[[T], S]]]] = []
+        self._subscribers: List[Tuple[Writable[S], Optional[Callable[[T_co], S]]]] = []
         self._triggers: List[Tuple[LogicHandler, bool]] = []
         self._pending_updates: Dict[int, Set[asyncio.Task]] = {}
 
-    async def __publish_write(self, subscriber: Writable[S], converter: Optional[Callable[[T], S]], value: T,
+    async def __publish_write(self, subscriber: Writable[S], converter: Optional[Callable[[T_co], S]], value: T_co,
                               origin: List[Any], use_pending: bool):
         try:
             await subscriber.write(converter(value) if converter else value, origin + [self])  # type: ignore
@@ -169,7 +172,7 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
             if use_pending:
                 self._pending_updates[id(subscriber)].discard(asyncio.current_task())
 
-    async def __publish_trigger(self, target: LogicHandler, value: T,
+    async def __publish_trigger(self, target: LogicHandler, value: T_co,
                                 origin: List[Any], use_pending: bool):
         try:
             await target(value, origin + [self])
@@ -179,7 +182,7 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
             if use_pending:
                 self._pending_updates[id(target)].discard(asyncio.current_task())
 
-    def _publish(self, value: T, origin: List[Any]):
+    def _publish(self, value: T_co, origin: List[Any]):
         """
         Method to publish a new value to all subscribers and trigger all registered logic handlers.
 
@@ -225,7 +228,7 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
                 if not any(s is subscriber for s in origin):
                     asyncio.create_task(self.__publish_write(subscriber, converter, value, origin, False))
 
-    async def _publish_and_wait(self, value: T, origin: List[Any]):
+    async def _publish_and_wait(self, value: T_co, origin: List[Any]):
         """
         A coroutine to publish a new value to all subscribers and trigger all registered logic handlers and wait for
         this publishing to finish.
@@ -254,7 +257,7 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
         elif sync_jobs:
             await asyncio.gather(*sync_jobs)
 
-    def subscribe(self, subscriber: Writable[S], convert: Union[Callable[[T], S], bool] = False) -> None:
+    def subscribe(self, subscriber: Writable[S], convert: Union[Callable[[T_co], S], bool] = False) -> None:
         """
         Subscribe a writable object to this object to be updated, when this object publishes a new value.
 
@@ -270,7 +273,7 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
             *or* if ``convert`` is True but no type conversion is known to convert this object's type into the
             subscriber's type.
         """
-        converter: Optional[Callable[[T], S]]
+        converter: Optional[Callable[[T_co], S]]
         if callable(convert):
             converter = convert
         elif issubclass(self.type, subscriber.type):
@@ -328,15 +331,15 @@ class Subscribable(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
         return target
 
 
-class Reading(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
+class Reading(Connectable[T_con], Generic[T_con], metaclass=abc.ABCMeta):
     is_reading_optional: bool = True
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._default_provider: Optional[Tuple[Readable[S], Optional[Callable[[S], T]]]] = None
+        self._default_provider: Optional[Tuple[Readable[S], Optional[Callable[[S], T_con]]]] = None
 
-    def set_provider(self, provider: Readable[S], convert: Union[Callable[[S], T], bool] = False):
-        converter: Optional[Callable[[S], T]]
+    def set_provider(self, provider: Readable[S], convert: Union[Callable[[S], T_con], bool] = False):
+        converter: Optional[Callable[[S], T_con]]
         if callable(convert):
             converter = convert
         elif issubclass(provider.type, self.type):
@@ -348,7 +351,7 @@ class Reading(Connectable[T], Generic[T], metaclass=abc.ABCMeta):
                             .format(repr(provider), provider.type.__name__, repr(self), self.type.__name__))
         self._default_provider = (provider, converter)
 
-    async def _from_provider(self) -> Optional[T]:
+    async def _from_provider(self) -> Optional[T_con]:
         """
         Private method to be used by inheriting classes to read the current value from this object's default provider
         (via its :meth:`Readable.read` method) and convert it to this object's type, if necessary, using the registered
