@@ -10,10 +10,13 @@
 # specific language governing permissions and limitations under the License.
 
 import abc
+import asyncio
 import datetime
+import functools
 import math
 import operator
-from typing import Type, Generic, Any, Iterable, Callable, Union, Dict, Tuple
+import typing
+from typing import Type, Generic, Any, Iterable, Callable, Union, Dict, Tuple, List, Optional
 
 from . import conversion
 from .base import Readable, Subscribable, T, Connectable, Writable, S, LogicHandler, UninitializedError
@@ -50,10 +53,6 @@ class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
     * ``>=`` (ge)
 
     Additionally, the following methods are provided:
-
-    * :meth:`not`: To be used instead of the ``not`` operator (which cannot be overridden)
-    * :meth:`convert`: Creates a "type cast" expression, which converts this object's value to the specified value type,
-      using the default converter function from :mod:`shc.conversion`.
     """
     @staticmethod
     def __get_other_type(other: object) -> type:
@@ -176,34 +175,34 @@ class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
         else:
             raise TypeError("{} cannot be rounded".format(self))
 
-    def __eq__(self, other) -> "BinaryExpressionHandler":  # type: ignore
+    def __eq__(self, other) -> "BinaryExpressionHandler[bool]":  # type: ignore
         return BinaryExpressionHandler(bool, self, other, operator.eq)
 
-    def __ne__(self, other) -> "BinaryExpressionHandler":  # type: ignore
+    def __ne__(self, other) -> "BinaryExpressionHandler[bool]":  # type: ignore
         return BinaryExpressionHandler(bool, self, other, operator.eq)
 
-    def __lt__(self, other) -> "BinaryExpressionHandler":
+    def __lt__(self, other) -> "BinaryExpressionHandler[bool]":
         other_type = self.__get_other_type(other)
         if (self.type, other_type) in TYPES_LT_LE_GT_GE:
             return BinaryExpressionHandler(TYPES_LT_LE_GT_GE[(self.type, other_type)], self, other, operator.lt)
         else:
             return NotImplemented
 
-    def __le__(self, other) -> "BinaryExpressionHandler":
+    def __le__(self, other) -> "BinaryExpressionHandler[bool]":
         other_type = self.__get_other_type(other)
         if (self.type, other_type) in TYPES_LT_LE_GT_GE:
             return BinaryExpressionHandler(TYPES_LT_LE_GT_GE[(self.type, other_type)], self, other, operator.le)
         else:
             return NotImplemented
 
-    def __gt__(self, other) -> "BinaryExpressionHandler":
+    def __gt__(self, other) -> "BinaryExpressionHandler[bool]":
         other_type = self.__get_other_type(other)
         if (self.type, other_type) in TYPES_LT_LE_GT_GE:
             return BinaryExpressionHandler(TYPES_LT_LE_GT_GE[(self.type, other_type)], self, other, operator.gt)
         else:
             return NotImplemented
 
-    def __ge__(self, other) -> "BinaryExpressionHandler":
+    def __ge__(self, other) -> "BinaryExpressionHandler[bool]":
         other_type = self.__get_other_type(other)
         if (self.type, other_type) in TYPES_LT_LE_GT_GE:
             return BinaryExpressionHandler(TYPES_LT_LE_GT_GE[(self.type, other_type)], self, other, operator.ge)
@@ -216,11 +215,27 @@ class ExpressionBuilder(Connectable[T], metaclass=abc.ABCMeta):
     def or_(self, other) -> "BinaryCastExpressionHandler":
         return BinaryCastExpressionHandler(bool, self, other, operator.or_)
 
-    def not_(self) -> "UnaryExpressionHandler":
+    def not_(self) -> "UnaryExpressionHandler[bool]":
         return UnaryCastExpressionHandler(bool, self, operator.not_)
 
-    def convert(self, type_: Type[T]) -> "UnaryExpressionHandler":
-        converter = conversion.get_converter(self.type, type_)
+    def convert(self, type_: Type[S], converter: Optional[Callable[[T], S]] = None) -> "UnaryExpressionHandler[S]":
+        """
+        Returns an ExpressionHandler that wraps this object to convert values to another type using a given converter
+        function or the :func:`default converter <conversion.get_converter>` for those two types.
+
+        Example::
+
+            value = shc.Variable(float)
+            which_one = shc.Variable(bool)
+            # display_string shall be a str-typed expression object
+            display_string = expression.IfThenElse(which_one, value.EX.convert(str), "static string")
+
+        :param type_: The target type to convert new values to. This will also by the `type` attribute of the returned
+            ExpressionHandler object, such that it will be used for further
+        :param converter: An optional converter function to use instead of the default conversion from this object's
+            type to the given target type.
+        """
+        converter = converter or conversion.get_converter(self.type, type_)
         return UnaryExpressionHandler(type_, self, converter)
 
 
@@ -279,6 +294,7 @@ class ExpressionHandler(Readable[T], Subscribable[T], ExpressionBuilder, Generic
         class Wrapper(Readable[S]):
             def __init__(self, v: S):
                 self.v = v
+                self.type = type(v)
 
             async def read(self) -> S:
                 return self.v
@@ -342,6 +358,11 @@ class IfThenElse(ExpressionHandler, Generic[T]):
     be ``bool`` (or a *Connectable* with value type ``bool``), `then` and `otherwise` must be of the same type (reps.
     have the same value type). If the `condition` evaluates to True, this object evaluates to the value of `then`,
     otherwise it evaluates to the value of `otherwise`.
+
+    See also :class:`Multiplexer` for switching between more than two values using an integer control value.
+    If you only want enable and disable a single subscription dynamically (e.g. enable/disable the influence of a
+    :class:`TimerSwitch <shc.timer.TimerSwitch>` on a :class:`Variable <shc.variables.Variable>`), take a look at
+    :class:`shc.misc.BreakableSubscription` instead.
     """
     def __init__(self, condition: Union[bool, Readable[bool]], then: Union[T, Readable[T]],
                  otherwise: Union[T, Readable[T]]):
@@ -354,6 +375,84 @@ class IfThenElse(ExpressionHandler, Generic[T]):
         return (await self.then.read()) if (await self.condition.read()) else (await self.otherwise.read())
 
 
+class Multiplexer(Readable[T], Subscribable[T], ExpressionBuilder[T], Generic[T]):
+    """
+    A :class:`ExpressionHandler` that behaves as a multiplexer block with an integer control value and any number of
+    inputs from that the output value is chosen
+
+    The control object, the first argument of this class, needs to be a Readable (and optionally Subscribable)
+    object of int type. All other parameters are considered to be inputs. They must either be static values or
+    Readable (and optionally Subscribable) objects. All of them must be of the same value type (resp. be instances of
+    exactly that type). This type will also be the type of the Multiplexer itself.
+
+    The control signal selects which of the inputs is passed through and is the provided value of the Multiplexer. E.g.,
+    when the control object has the value 0, *reading* from the Multiplexer returns the first input's current value
+    (i.e. the value of the object given as the second argument); when the control object has the value 1, the second
+    input's value is returned, and so on. If the control value is out of range of the available inputs, *reading*
+    returns an :class:`UninitializedError`.
+
+    The Multiplexer publishes a value update whenever an update from the control object or the currently selected
+    input is received – as long as they are Subscribable.
+
+    See also :class:`IfThenElse` for simple cases with only two values and a boolean control value.
+    If you only want enable and disable a single subscription dynamically (e.g. enable/disable the influence of a
+    :class:`TimerSwitch <shc.timer.TimerSwitch>` on a :class:`Variable <shc.variables.Variable>`), take a look at
+    :class:`shc.misc.BreakableSubscription` instead.
+    """
+    _synchronous_publishing = True
+
+    def __init__(self, control: Readable[int], *inputs: Union[T, Readable[T]]):
+        super().__init__()
+        if not inputs:
+            raise ValueError("Multiplexer must have at least one input")
+        self.inputs: List[Readable[T]] = [input_ if isinstance(input_, Readable)
+                                          else ExpressionHandler._wrap_static_value(input_)
+                                          for input_ in inputs]
+        self.type: Type[T] = self.inputs[0].type
+        for i, input_ in enumerate(self.inputs):
+            if input_.type != self.type:
+                raise ValueError("Type of input {} ({}) does not match: {} instead of {}"
+                                 .format(i, input, input_.type, self.type))
+            if isinstance(input_, Subscribable):
+                input_.trigger(self._new_value, synchronous=True)
+
+        self.control = control
+        if isinstance(control, Subscribable):
+            control.trigger(self._index_change, synchronous=True)
+
+    async def read(self) -> T:
+        try:
+            current_index = await self.control.read()
+            current_input = self.inputs[current_index]
+        except IndexError:
+            # We "ignore" index errors here. They may have been logged by _index_change() before
+            raise UninitializedError()
+        return await current_input.read()
+
+    async def _new_value(self, value: T, origin: List[Any]) -> None:
+        try:
+            current_index = await self.control.read()
+            current_input = self.inputs[current_index]
+        except UninitializedError:
+            # We default to False, if the control variable is not initialized yet
+            return
+        except IndexError:
+            # We ignore index errors here. They may have been logged by _index_change() before
+            return
+        if current_input is origin[-1]:
+            await self._publish_and_wait(value, origin)
+
+    async def _index_change(self, current_index: int, origin: List[Any]) -> None:
+        try:
+            current_input = self.inputs[current_index]
+        except IndexError:
+            raise ValueError("Index {} is out of range of available inputs of {}".format(current_index, self))
+        try:
+            await self._publish_and_wait(await current_input.read(), origin)
+        except UninitializedError:
+            pass
+
+
 def not_(a) -> Union[bool, UnaryCastExpressionHandler[bool]]:
     """
     Create a :class:`ExpressionHandler` that wraps a *Connectable* or static value and evaluates to ``not x`` for the
@@ -392,6 +491,62 @@ def or_(a, b) -> Union[bool, BinaryCastExpressionHandler[bool]]:
     return BinaryCastExpressionHandler(bool, a, b, operator.or_)
 
 
+# This is just a hack to make type checkers happy with the signature of the __init__ method of return types by
+# expression
+class ExpressionFunctionHandler(ExpressionHandler[T], Generic[T]):
+    def __init__(self, *args: Any) -> None: ...
+
+
+def expression(func: Callable[..., T]) -> Type[ExpressionFunctionHandler[T]]:
+    """
+    A decorator to turn any simple function into an :class:`ExpressionHandler` class to be used in SHC expressions
+
+    This way, custom functions can be used in SHC expression in an intuitive way::
+
+        @expression
+        def invert(a: float) -> float:
+            return 1 / a
+
+        var1 = Variable(float)
+        var2 = Variable(float).connect( invert(var1) )
+
+    The decorator turns the function into a class, such that calling it will return an instance of that class instead of
+    evaluating the function. The resulting object is a *Readable* and *Subscribable* object, that evaluates the
+    original function with the current values of the given arguments on every *read()* and every value update. The
+    arguments passed to each instance must either be *Readable* (and optionally *Subscribable*) objects, providing
+    values of the expected argument type, or simply static objects of the expected type.
+
+    The wrapped function must be synchronous (not async) and have a type annotation for its return type. The type
+    annotation must refer to a runtime type, which can be used for dynamic type checking (i.e. no subscripted generic
+    type; use `list` instead of `List[int]` and similar). This return type annotation is also the type of the resulting
+    ExpressionHandler objects.
+
+    Currently only positional arguments (no keyword arguments) are supported for the functions.
+    """
+    class WrapHandler(ExpressionHandler[T]):
+        def __init__(self, *args):
+            self.args = [self._wrap_static_value(a) for a in args]
+            type_ = typing.get_type_hints(func).get('return')
+            if not type_:
+                raise TypeError("@expression requires the wrapped function to have a return type annotation")
+            # Check if the type annotation is valid for runtime type checking
+            try:
+                isinstance(None, type_)
+            except TypeError as e:
+                raise TypeError("@expression requires the wrapped function to have a return type annotation") from e
+
+            super().__init__(type_, args)
+
+        async def evaluate(self) -> T:
+            return func(*(await asyncio.gather(*(a.read() for a in self.args))))
+
+        def __repr__(self) -> str:
+            return "@expression[{}({})]".format(func.__name__, repr(self.args))
+
+    functools.update_wrapper(WrapHandler, func, updated=())
+    return WrapHandler  # type: ignore
+
+
 # #################### #
 # Type Inference Rules #
 # #################### #
