@@ -17,7 +17,7 @@ import logging
 from typing import List, Any, Generic, Type, Callable, Awaitable, Optional, Tuple, Union, Dict, Deque
 
 from paho.mqtt.client import MQTTMessage, MQTTv311  # type: ignore
-from asyncio_mqtt import Client, MqttError  # type: ignore
+from asyncio_mqtt import Client, MqttError, ProtocolVersion  # type: ignore
 from paho.mqtt.matcher import MQTTMatcher  # type: ignore
 
 from ._helper import SupervisedClientInterface
@@ -55,7 +55,7 @@ class MQTTClientInterface(SupervisedClientInterface):
     :param username: MQTT broker login username (no login is performed if None)
     :param password: MQTT broker login password (no login is performed if None)
     :param client_id: MQTT client name (if None, it is auto-generated by the Paho MQTT client)
-    :param protocol: MQTT protocol version (from Paho MQTT library), defaults to MQTT 3.11
+    :param protocol: MQTT protocol version (from asyncio_mqtt library), defaults to MQTT 3.11
     :param auto_reconnect: If True (default), the client tries to reconnect automatically on connection errors with
         exponential backoff (1 * 1.25^n seconds sleep). Otherwise, the complete SHC system is shut down, when a
         connection error occurs.
@@ -65,11 +65,12 @@ class MQTTClientInterface(SupervisedClientInterface):
         retried and will shutdown the SHC application on failure, even if `auto_reconnect` is True.
     """
     def __init__(self, hostname: str = 'localhost', port: int = 1883, username: Optional[str] = None,
-                 password: Optional[str] = None, client_id: Optional[str] = None, protocol: int = MQTTv311,
+                 password: Optional[str] = None, client_id: Optional[str] = None,
+                 protocol: Union[ProtocolVersion, int] = ProtocolVersion.V311,
                  auto_reconnect: bool = True, failsafe_start: bool = False) -> None:
         super().__init__(auto_reconnect, failsafe_start)
         self.client = Client(hostname, port, username=username, password=password, client_id=client_id,
-                             protocol=protocol)
+                             protocol=ProtocolVersion(protocol))
         self.matcher = MQTTMatcher()
         self.subscribe_topics: Dict[str, int] = {}  #: dict {topic: qos} for subscribing
         self.run_task: Optional[asyncio.Task] = None
